#!/usr/bin/env python

# Copyright 2017 Jose Nelson Amaral
# Copyright 2017 Elliot Colp
# 
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
# 
# 1. Redistributions of source code must retain the above copyright notice, this
# list of conditions and the following disclaimer.
# 
# 2. Redistributions in binary form must reproduce the above copyright notice,
# this list of conditions and the following disclaimer in the documentation
# and/or other materials provided with the distribution.
# 
# 3. Neither the name of the copyright holder nor the names of its contributors
# may be used to endorse or promote products derived from this software without
# specific prior written permission.
# 
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
# FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
# DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
# SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
# CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
# OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

"""
This script takes random chess positions from the positions.epd file and
produces a workload for the deepsjeng_r benchmark.

Usage: ./gen-input.py [options]

This script's output will be placed in ./<workload-name>
"""

import sys, os, argparse, shutil, subprocess
import random
import math
    
# Load the positions
with open("positions.epd", "r") as f:
    positions = f.readlines()

# Parse arguments
parser = argparse.ArgumentParser(usage="./gen-input.py [options]",
description="""
Randomly choose positions from positions.epd to create workloads for deepsjeng_r.
See docstring at top of gen-input.py for more information.
""",
formatter_class=argparse.ArgumentDefaultsHelpFormatter)
parser.add_argument("--name", "-n", help="The workload's name. If not given, the name will be the lowest number from 000 to 999 which doesn't already exist in this directory.",
                    default=argparse.SUPPRESS)
parser.add_argument("--count", "-c", help="The number of board positions to include.",
                    default=10, type=int)
parser.add_argument("--depth", "-d", help="The minimum and maximum search depths.",
                    nargs=2, default=[15, 20], type=int, metavar=("MIN", "MAX"))
parser.add_argument("--seed", "-s", help="The seed to feed the random number generator before creating the workload. If not given, system time is used.",
                    default=argparse.SUPPRESS)

parsedArgs, parsedRest = parser.parse_known_args()
    
# Get min/max depth
minDepth, maxDepth = parsedArgs.depth

# Get name
if "name" not in parsedArgs:
    # Find the next number
    for i in xrange(999):
        iStr = "{:03}".format(i)
        
        if not os.path.isdir(iStr):
            wkldName = iStr
            break
else:
    wkldName = parsedArgs.name

# Read the workload name (stripping newline)
wkldDir = "./" + wkldName
inputDir = wkldDir + "/input"
outputDir = wkldDir + "/output"

print("\nBuilding workload \"" + wkldName + "\"")

# Remove old data
if os.path.isdir(wkldDir):
    resp = raw_input("Directory \"" + wkldDir + "\" already exists. Overwrite? ")
    if (resp not in set(["yes", "y"])):
        print("Aborting")
        exit()
    else:
        shutil.rmtree(wkldDir)
        print()
    
# Set up directories
os.makedirs(inputDir)
os.makedirs(outputDir)

# Write dummy reftime/refpower files
with open(wkldDir + "/reftime", "w") as f:
    f.write(wkldName + "\n2\n")
    
with open(wkldDir + "/refpower", "w") as f:
    f.write(wkldName + "\n2\n")

# Seed the PRNG
if "seed" in parsedArgs:
    random.seed(parsedArgs.seed)

# Create the input
inputFile = "{}/{}.txt".format(inputDir, wkldName)
with open(inputFile, "w") as f:
    for i in range(parsedArgs.count):
        # Choose a random position without replacement
        pos = random.choice(positions)
        f.write(pos)
        positions.remove(pos)
        
        # Choose a random length
        f.write(str(random.randint(minDepth, maxDepth)))
        f.write("\n")

'''args = ["./deepsjeng_r", inputFile]

print("Running benchmark")
benchOut = subprocess.check_output(args, universal_newlines=True)

# Store output
print("Storing data")
outFilename = outputDir + "/" + wkldName + ".out"
with open(outFilename, "w") as outFile:
    outFile.write(str(benchOut))
        
print("Complete!\n".format(wkldName))'''
